// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/18AISS8AO8/18AISS8AO8_1.x_GSC_DN/driver/main.h $
// $Rev: 52620 $
// $Date: 2023-03-10 14:56:23 -0600 (Fri, 10 Mar 2023) $

// 18AISS8AO8: Device Driver: header file

#ifndef	__MAIN_H__
#define	__MAIN_H__

#include "18aiss8ao8.h"

#define	DEV_BAR_SHOW						0
#define	DEV_METRICS_SHOW					0
#define	DEV_PCI_ID_SHOW						0
#define	DEV_SUPPORTS_IRQ
#define	DEV_SUPPORTS_READ
#define	DEV_SUPPORTS_WAIT
#define	DEV_SUPPORTS_WRITE
#define	GSC_ALT_DEV_GET(a)					(a)
#define	GSC_ALT_STRUCT_GET(i,d)				(d)
#define	GSC_ALT_STRUCT_T					dev_data_t
#define	GSC_DEVS_PER_BOARD					1

typedef struct _dev_data_t					dev_data_t;

#include "gsc_main.h"



// macros *********************************************************************

#define	DEV_MODEL							"18AISS8AO8"	// Upper case form of the below.
#define	DEV_NAME							"18aiss8ao8"	// MUST AGREE WITH AISS8AO8_BASE_NAME

#define	DEV_VERSION							"1.4"			// FOR DEVICE SPECIFIC CODE ONLY!
// 1.4	Updated to support the 6.x series kernel.
//		Updated macros for consistency.
// 1.3	Updated comments for consistency.
//		Standardized processing of the initialization IOCTL service.
//		Standardized processing of the autocalibration IOCTL service.
//		Updated for additional kernel support.
//		Updated per file reorganization.
//		Updated per DMA code reorganization.
// 1.2	Added support for the 5.x kernel series.
//		Updated for porting to Windows.
//		Updated I/O bytes available count for consistency.
//		Standardized various IOCTL system log messages.
//		Standardized processing of the initialization IOCTL service.
//		Standardized processing of the autocalibration IOCTL service.
// 1.1	Added support for the 5.x kernel series.
//		White space cleanup.
//		Open succeeds even if initilization fails.
//		Added query option for DMDMA support.
//		Added support for the AI Buffer Underflow feature.
//		Added Endianness support.
//		Added AISS8AO8_QUERY_AI_UNDERFLOW.
//		Added AISS8AO8_IOCTL_RX_IO_UNDERFLOW.
//		Added AISS8AO8_IOCTL_AI_BUF_UNDERFLOW.
// 1.0	Initial release.

// I/O services
#define	DEV_IO_STREAM_QTY					(DEV_IO_RX_STREAM_QTY + DEV_IO_TX_STREAM_QTY)
#define	DEV_IO_RX_STREAM_QTY				1
#define	DEV_IO_TX_STREAM_QTY				1

#define	GSC_READ_PIO_WORK_32_BIT
#define	GSC_WRITE_PIO_WORK_32_BIT

// WAIT services
#define	DEV_WAIT_GSC_ALL					AISS8AO8_WAIT_GSC_ALL
#define	DEV_WAIT_ALT_ALL					AISS8AO8_WAIT_ALT_ALL
#define	DEV_WAIT_IO_ALL						AISS8AO8_WAIT_IO_ALL



// data types *****************************************************************

struct _dev_io_t
{
	// Initialized by open and IOCTL services (initialize and/or service specific).

	s32					dma_threshold;		// Do BMDMA after we've got this many samples.
	s32					io_mode;			// PIO, DMA, DMDMA
	s32					overflow_check;		// Check overflow when reading?
	s32					pio_threshold;		// Use PIO if samples <= this.
	s32					timeout_s;			// I/O timeout in seconds.
	s32					underflow_check;	// Check underflow when reading?

	// Initialized by I/O service at time of use.

	int					abort;				// Also set via IOCTL service.
	gsc_dma_ch_t*		dma_channel;		// Use this channel for DMA.
	int					non_blocking;		// Is this non-blocking I/O?

	// Initialized by device specific I/O create code.

	int					bytes_per_sample;	// Sample size in bytes.
	u32					io_reg_offset;		// Offset of device's I/O FIFO.
	VADDR_T				io_reg_vaddr;		// Address of device's I/O FIFO.

	void				(*dev_io_sw_init)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	void				(*dev_io_close)			(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	void				(*dev_io_open)			(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	int					(*dev_io_startup)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	long				(*dev_pio_available)	(GSC_ALT_STRUCT_T* alt, dev_io_t* io, size_t count);
	long				(*dev_bmdma_available)	(GSC_ALT_STRUCT_T* alt, dev_io_t* io, size_t count);
	long				(*dev_dmdma_available)	(GSC_ALT_STRUCT_T* alt, dev_io_t* io, size_t count);
	long				(*dev_pio_xfer)			(GSC_ALT_STRUCT_T* alt, dev_io_t* io, const os_mem_t* mem, size_t count, os_time_tick_t st_end);
	long				(*dev_bmdma_xfer)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io, const os_mem_t* mem, size_t count, os_time_tick_t st_end);
	long				(*dev_dmdma_xfer)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io, const os_mem_t* mem, size_t count, os_time_tick_t st_end);

	struct
	{
		u32				abort;		// ...WAIT_IO_XXX_ABORT
		u32				done;		// ...WAIT_IO_XXX_DONE
		u32				error;		// ...WAIT_IO_XXX_ERROR
		u32				timeout;	// ...WAIT_IO_XXX_TIMEOUT
	} wait;

	// Initialize by GSC commone create code.

	os_mem_t			mem;				// I/O buffer.
	os_sem_t			sem;				// Only one Tx or Rx at a time.
};

struct _dev_data_t
{
	os_pci_t			pci;			// The kernel PCI device descriptor.
	os_data_t			os;				// OS specific data.
	os_spinlock_t		spinlock;		// Control ISR access.
	os_sem_t			sem;			// Control thread access.
	gsc_dev_type_t		board_type;		// Corresponds to basic device type.
	const char*			model;			// Base model number in upper case.
	int					board_index;	// Index of the device being accessed.
	int					users;			// Number of currently active open() requests.
	int					share;			// Were we opened in shared mode?

	gsc_bar_t			bar;			// device register mappings
	gsc_dma_t			dma;			// For DMA based I/O.
	gsc_irq_t			irq;			// For interrut support.
	gsc_wait_node_t*	wait_list;

	struct
	{					// This is for streaming I/O
		dev_io_t		rx;				// For read operations.
		dev_io_t		tx;				// For write operations.
		dev_io_t*		io_streams[DEV_IO_STREAM_QTY];
	} io;

	struct
	{
		VADDR_T			plx_intcsr_32;	// Interrupt Control/Status Register
		VADDR_T			plx_dmaarb_32;	// DMA Arbitration Register
		VADDR_T			plx_dmathr_32;	// DMA Threshold Register

		VADDR_T			gsc_acr_32;		// Assembly Configuration Register
		VADDR_T			gsc_bcr_32;		// Board Control Register
		VADDR_T			gsc_boor_32;	// Buffered Output Operations Register
		VADDR_T			gsc_clsr_32;	// Current Loop Selection Register

		VADDR_T			gsc_diopr_32;	// Digital I/O Port Register

		VADDR_T			gsc_ibbsr_32;	// Input Burst Block Size Register
		VADDR_T			gsc_ibdr_32;	// Input Buffer Data Register
		VADDR_T			gsc_ibsr_32;	// Input Buffer Size Register
		VADDR_T			gsc_ibtr_32;	// Input Buffer Threshold Register
		VADDR_T			gsc_icr_32;		// Input Configuration Register

		VADDR_T			gsc_obdr_32;	// Output Buffer Data Register
		VADDR_T			gsc_obsr_32;	// Output Buffer Size Register
		VADDR_T			gsc_obtr_32;	// Output Buffer Threshold Register
		VADDR_T			gsc_ocr_32;		// Output Configuration Register

		VADDR_T			gsc_psr_32;		// Primary Status Register

		VADDR_T			gsc_ragr_32;	// Rate-A Generator Register
		VADDR_T			gsc_rbgr_32;	// Rate-B Generator Register
		VADDR_T			gsc_rcgr_32;	// Rate-C Generator Register
		VADDR_T			gsc_rdgr_32;	// Rate-D Generator Register

	} vaddr;

	struct
	{
		s32				ai_chans_max;		// Maximum input channels supported.
		s32				ai_chan_qty;		// The number of input channels on the device.
		s32				ai_underflow;		// Is the Input Buffer Underflow feature supported?
		s32				ao_chans_max;		// Maximum output channels supported.
		s32				ao_chan_qty;		// The number of output channels on the device.
		s32				auto_cal_ms;		// Maximum ms for auto-cal

		s32				dmdma;				// Does the firmware support Demand Mode DMA?

		s32				fifo_size_rx;		// Size of Rx FIFO - not the fill level.
		s32				fifo_size_tx;		// Size of Tx FIFO - not the fill level.
		s32				filter_a;			// Installed Filter A option.
		s32				filter_b;			// Installed Filter A option.
		s32				fsamp_max;			// The maximum Fsamp rate per channel.
		s32				fsamp_min;			// The minimum Fsamp rate per channel.

		u32				gsc_acr_32;			// Assembly Configuration Register

		s32				initialize_ms;		// Maximum ms for initialize

		s32				master_clock;		// Master clock frequency

		s32				ndiv_max;			// Maximum Ndiv value.
		s32				ndiv_min;			// Minimum Ndiv value.

	} cache;
};



// prototypes *****************************************************************

void		dev_io_close(dev_data_t* dev);
int			dev_io_create(dev_data_t* dev);
void		dev_io_destroy(dev_data_t* dev);
int			dev_io_open(dev_data_t* dev);
dev_io_t*	dev_io_read_select(dev_data_t* dev, size_t count);
dev_io_t*	dev_io_write_select(dev_data_t* dev, size_t count);
int			dev_irq_create(dev_data_t* dev);
void		dev_irq_destroy(dev_data_t* dev);
int			dev_read_create(dev_data_t* dev, dev_io_t* io);
int			dev_write_create(dev_data_t* dev, dev_io_t* io);

int			initialize_ioctl(dev_data_t* dev, void* arg);



#endif
