// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/18AISS8AO8/18AISS8AO8_1.x_GSC_DN/samples/aout/perform.c $
// $Rev: 51970 $
// $Date: 2022-10-26 10:40:26 -0500 (Wed, 26 Oct 2022) $

// 18AISS8AO8: Sample Application: source file

#include "main.h"



// variables ******************************************************************

static	u32		_data[128 * 1024];
static	long	_loops;	// Number of times to send a block of data.
static	long	_qty;	// Number of samples to send per block.



//*****************************************************************************
static int _io_write(const args_t* args, int errs)
{
	struct timeval	begin;
	struct timeval	end;
	long			loop;
	char*			ptr;
	int				ret;
	int				send;
	int				sent	= 0;
	long			sps;
	s32				status;
	long long		total	= 0;
	long			us;

	gsc_label("Writing Data");

	for (;;)	// A convenience loop.
	{
		if (errs)
		{
			errs	= 0;
			printf("SKIPPED  (due to errors)\n");
			break;
		}

		errs	= 0;

		// Write the data to the device.
		gettimeofday(&begin, NULL);

		for (loop = 0; loop < _loops; loop++)
		{
			send	= _qty * 4;
			ptr		= (char*) _data;

			for (;;)
			{
				sent	= aiss8ao8_write(args->fd, ptr, send);

				if (sent < 0)
					break;

				total	+= sent;

				if (sent == send)
					break;

				ptr		+= sent;
				send	-= sent;
			}

			if (sent < 0)
				break;
		}

		// Wait for the output buffer to empty.

		for (;;)
		{
			ret		= aiss8ao8_ioctl(args->fd, AISS8AO8_IOCTL_AO_BUF_EMPTY, &status);
			errs	+= ret ? 1 : 0;

			if ((errs) || (status == AISS8AO8_AO_BUFFER_EMPTY_YES))
				break;
		}

		// Compute the results.

		gettimeofday(&end, NULL);

		if (sent < 0)
		{
			errs	= 1;
			printf("FAIL <---  (aiss8ao8_write() error)\n");
		}
		else
		{
			if (sent == 0)
				printf("FAIL <---  (I/O timeout, ");
			else
				printf("PASS  (");

			us	= (end.tv_sec - begin.tv_sec) * 1000000L
				+ (end.tv_usec - begin.tv_usec);
			sps	= (total * 10000000L / 4 / us + 5) / 10;

			gsc_label_long_comma(total);
			printf(" Bytes, %ld.%06ld Seconds, ", us / 1000000, us % 1000000);
			gsc_label_long_comma(sps);
			printf(" S/S)\n");
		}

		break;
	}

	return(errs);
}



//*****************************************************************************
static int _init_data(const args_t* args, int chans)
{
	#define	PIE		(3.1415926535)

	u32*		dest;
	int			i;
	long		period_len;	// The length of the pattern in samples.
	long		period_tot;	// Pattern samples total for all channels.
	long		period;		// Pattern length in x units.
	int			samp;
	float		samp_dat;	// Total sample size of the data buffer.
	float		samp_sec;	// Samples Per Second for all active channels
	float		samp_tot;	// Samples Total for all active channels
	const char*	units;

	gsc_label("Initializing Data");
	memset(_data, 0, sizeof(_data));
	samp_sec	= ((float) args->rate) * chans;
	samp_tot	= ((float) args->rate) * chans * args->seconds;
	samp_dat	= sizeof(_data) / 4;

	period_len	= (args->rate <= 100) ? args->rate				//      10-      100 ->    10-   100
				: (args->rate <= 1000) ? (args->rate / 2)		//     101-    1,000 ->    50-   500
				: (args->rate <= 10000) ? (args->rate / 5)		//   1,001-   10,000 ->   200- 2,000
				: (args->rate <= 100000) ? (args->rate / 10)	//  10,001-  100,000 -> 1,000-10,000
				: (args->rate / 100);							// 100,001-1,000,000 -> 1,000-10,000
	period_tot	= period_len * chans;

	if ((args->rate) && (period_len))
		period	= (long) (1000000000. / args->rate * period_len);
	else
		period	= 0;

	units		= "ns";

	if ((period % 1000) == 0)
	{
		period	/= 1000;
		units	= "us";
	}

	if ((period % 1000) == 0)
	{
		period	/= 1000;
		units	= "ms";
	}

	if (samp_tot <= SIZEOF_ARRAY(_data))
	{
		// We can get the entire image in the buffer.
		_loops	= 1;
		_qty	= (long) samp_tot;
	}
	else if (samp_sec <= SIZEOF_ARRAY(_data))
	{
		// We can get one second of the image in the buffer.
		_loops	= args->seconds;
		_qty	= (long) samp_sec;
	}
	else if (period_tot <= SIZEOF_ARRAY(_data))
	{
		// We can get one period of the image in the buffer.
		_loops	= (long) (args->seconds * (samp_sec / period_tot));
		_qty	= period_tot;
	}
	else
	{
		// We can get only a fraction of a period of the image in the buffer.
		_qty	= (long) ((samp_dat / chans) * chans);
		_loops	= (long) ((samp_tot + _qty - 1) / _qty);
	}

	// Initialize the data buffer.
	// The data is encoded in OFFSET BINARY format.
	dest	= _data;;

	for (i = 0, samp = 0; i < _qty; i += chans, samp++)
	{
		samp	= (samp < period_len) ? samp : 0;

		// 1st channel: sawtooth wave
		*dest++	= (u32) (0x3FFFF - (0x3FFFF * samp / (period_len - 1)));

		// 2nd channel: ramp wave
		*dest++	= (u32) (0x3FFFF * samp / (period_len - 1));

		// 3rd channel: square wave
		*dest++	= (samp < (period_len / 2)) ? 0x3FFFF : 0;

		// 4th channel: sine wave
		*dest++	= ((long) 0x1FFFF) * (1 + sin((2 * PIE / period_len) * samp));
	}

	printf(	"DONE  (pattern length: %ld samples, %ld %s)\n",
			period_len,
			period,
			units);
	return(0);
}



//*****************************************************************************
int perform_tests(const args_t* args)
{
	int	errs	= 0;
	s32	dmdma	= 0;
	s32	io_mode	= args->io_mode;
	s32	rate;

	errs	+= aiss8ao8_config_ao(args->fd, -1, 1, args->rate);

	gsc_label("Analog Output");
	printf("\n");
	gsc_label_level_inc();

	if (io_mode == GSC_IO_MODE_DMDMA)
	{
		errs	= aiss8ao8_query(args->fd, -1, 1, AISS8AO8_QUERY_DMDMA, &dmdma);

		if (dmdma == 0)
		{
			gsc_label("NOTICE");
			printf("Using BMDMA as DMDMA is not supported. <---\n");
			io_mode	= GSC_IO_MODE_PIO;
		}
	}

	errs	+= aiss8ao8_ao_output_mode	(args->fd, -1, 1, AISS8AO8_AO_OUTPUT_MODE_SIMUL, NULL);
	errs	+= aiss8ao8_ao_chan_sel		(args->fd, -1, 1, 0xF, NULL);
	errs	+= aiss8ao8_fsamp_ao_report	(args->fd, -1, 1, -1, &rate);
	errs	+= aiss8ao8_tx_io_mode		(args->fd, -1, 1, io_mode, NULL);
	_init_data(args, 4);
	errs	+= aiss8ao8_ao_buf_over_fram(args->fd, -1, 1, AISS8AO8_BUF_OVERFLOW_CLEAR, NULL);
	errs	+= aiss8ao8_ao_buf_over_data(args->fd, -1, 1, AISS8AO8_BUF_OVERFLOW_CLEAR, NULL);
	errs	+= _io_write(args, errs);
	errs	+= aiss8ao8_ao_buf_over_fram(args->fd, -1, 1, AISS8AO8_BUF_OVERFLOW_TEST, NULL);
	errs	+= aiss8ao8_ao_buf_over_data(args->fd, -1, 1, AISS8AO8_BUF_OVERFLOW_TEST, NULL);

	gsc_label_level_dec();
	return(errs);
}



