// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/18AISS8AO8/18AISS8AO8_1.x_GSC_DN/samples/rxrate/perform.c $
// $Rev: 52606 $
// $Date: 2023-03-10 14:42:25 -0600 (Fri, 10 Mar 2023) $

// 18AISS8AO8: Sample Application: source file

#include "main.h"



// macros *********************************************************************

#define	_2M		(2 * 1024 * 1024)	// size of driver's transfer buffer



// variables ******************************************************************

static	u8	_rx[_2M];



//*****************************************************************************
static int _io_mode_select(const args_t* args)
{
	s32	dmdma	= 0;
	int	errs	= 0;

	errs	+= aiss8ao8_query(args->fd, -1, 1, AISS8AO8_QUERY_DMDMA, &dmdma);

	if ((args->io_mode == GSC_IO_MODE_DMDMA) && (dmdma == 0))
	{
		gsc_label_suffix(NULL, NULL);
		printf("Reverting to BMDMA since DMDMA is not supported.\n");
		errs	+= aiss8ao8_tx_io_mode(args->fd, -1, 1, GSC_IO_MODE_BMDMA, NULL);
	}
	else
	{
		errs	+= aiss8ao8_tx_io_mode(args->fd, -1, 1, args->io_mode, NULL);
	}

	return(errs);

}



//*****************************************************************************
static int _rate_test(const args_t* args, long get)
{
	struct timeval	begin;
	int				errs	= 0;
	long			got;
	long long		limit	= (long long) args->rx_mb * 1000000L;
	struct timeval	minimum;
	struct timeval	now;
	long long		sps;
	long long		total	= 0;
	long			us;

	gsc_label("Reading");

	gettimeofday(&begin, NULL);
	minimum.tv_sec	= begin.tv_sec + args->seconds;
	minimum.tv_usec	= begin.tv_usec;

	for (;;)
	{
		got	= aiss8ao8_read(args->fd, _rx, get);

		if (got < 0)
		{
			errs++;
			printf("FAIL <--- (read");
			printf(", total so far %lld", total);
			printf(", requested %ld bytes", (long) get);
			printf(", error %ld)\n", got);
			break;
		}

		gettimeofday(&now, NULL);
		total	+= got;

		if (total < limit)
			continue;

		if (now.tv_sec < minimum.tv_sec)
			continue;

		if ((now.tv_sec == minimum.tv_sec) && (now.tv_usec < minimum.tv_usec))
			continue;

		us	= (now.tv_sec - begin.tv_sec) * 1000000L
			+ (now.tv_usec - begin.tv_usec);
		sps	= (total * 10000000L / 4 / us + 5) / 10;

		printf(	"PASS  (");
		gsc_label_long_comma(total / 4);
		printf(" Samples, %ld.%06ld Seconds, ", us / 1000000, us % 1000000);
		gsc_label_long_comma(sps);
		printf(" S/S)\n");
		break;
	}

	return(errs);
}



//*****************************************************************************
int perform_tests(const args_t* args)
{
	int		errs	= 0;
	long	get		= sizeof(_rx);
	s32		fsamp;

	errs	+= aiss8ao8_query			(args->fd, -1, 1, AISS8AO8_QUERY_FSAMP_MAX, &fsamp);
	errs	+= aiss8ao8_config_ai		(args->fd, -1, 1, fsamp);
	errs	+= _io_mode_select(args);
	errs	+= aiss8ao8_rx_io_overflow	(args->fd, -1, 1, AISS8AO8_IO_OVERFLOW_IGNORE, NULL);
	errs	+= aiss8ao8_rx_io_underflow	(args->fd, -1, 1, AISS8AO8_RX_IO_UNDERFLOW_IGNORE, NULL);

	gsc_label ("Read Rate");
	printf("(%d second minimum, %ld MB minimum)\n", args->seconds, (long) args->rx_mb);
	gsc_label_level_inc();

	errs	+= aiss8ao8_ai_buf_clear	(args->fd, -1, 1);
	errs	+= aiss8ao8_ai_buf_overflow	(args->fd, -1, 1, AISS8AO8_BUF_OVERFLOW_CLEAR, NULL);
	errs	+= aiss8ao8_ai_buf_underflow(args->fd, -1, 1, AISS8AO8_AI_BUF_UNDERFLOW_CLEAR, NULL);

	errs	+= _rate_test(args, get);

	errs	+= aiss8ao8_ai_buf_overflow	(args->fd, -1, 1, AISS8AO8_BUF_OVERFLOW_TEST, NULL);
	errs	+= aiss8ao8_ai_buf_underflow(args->fd, -1, 1, AISS8AO8_AI_BUF_UNDERFLOW_TEST, NULL);

	gsc_label_level_dec();
	return(errs);
}



