// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/18AISS8AO8/18AISS8AO8_1.x_GSC_DN/samples/signals/perform.c $
// $Rev: 50418 $
// $Date: 2022-02-17 09:17:46 -0600 (Thu, 17 Feb 2022) $

// 18AISS8AO8: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _continuous_signals(const args_t* args)
{
	u32	buf[1];
	int	errs	= 0;
	s32	fsamp	= 40000;
	s32	ndiv;
	int	sent;
	s32	warp;

	gsc_label("Analog Input Clock");
	printf("\n");
	gsc_label_level_inc();
	// This generates a period of about 25us.
	errs	+= aiss8ao8_fsamp_ai_compute(args->fd, -1, 1, fsamp, &ndiv, &warp, NULL);
	errs	+= aiss8ao8_ai_clk_trig_out	(args->fd, -1, 1, AISS8AO8_AI_CLK_TRIG_OUT_DRIVE, NULL);
	errs	+= aiss8ao8_ai_enable		(args->fd, -1, 1, AISS8AO8_AIO_ENABLE_YES, NULL);
	errs	+= aiss8ao8_gen_a_enable	(args->fd, -1, 1, AISS8AO8_GEN_ENABLE_YES, NULL);
	errs	+= aiss8ao8_warp_mode		(args->fd, -1, 1, warp, NULL);
	errs	+= aiss8ao8_gen_a_ndiv		(args->fd, -1, 1, ndiv, NULL);
	errs	+= aiss8ao8_fsamp_ai_report	(args->fd, -1, 1, -1, NULL);
	gsc_label_level_dec();

	gsc_label("Analog Input Trigger");
	printf("\n");
	gsc_label_level_inc();
	// This generates a period of about 50us.
	errs	+= aiss8ao8_ai_burst_size	(args->fd, -1, 1, 1, NULL);
	errs	+= aiss8ao8_ai_burst_enable	(args->fd, -1, 1, AISS8AO8_BURST_ENABLE_YES, NULL);
	errs	+= aiss8ao8_gen_b_ndiv		(args->fd, -1, 1, ndiv * 2, NULL);
	errs	+= aiss8ao8_gen_b_enable	(args->fd, -1, 1, AISS8AO8_GEN_ENABLE_YES, NULL);
	gsc_label_level_dec();

	gsc_label("Analog Output Trigger");
	printf("\n");
	gsc_label_level_inc();
	// This generates a period of about 75us.
	errs	+= aiss8ao8_ao_burst_enable	(args->fd, -1, 1, AISS8AO8_BURST_ENABLE_YES, NULL);
	errs	+= aiss8ao8_gen_d_ndiv		(args->fd, -1, 1, ndiv * 2 * 3 / 2, NULL);
	errs	+= aiss8ao8_gen_d_enable	(args->fd, -1, 1, AISS8AO8_GEN_ENABLE_YES, NULL);
	gsc_label_level_dec();

	gsc_label("Analog Output Clock");
	printf("\n");
	gsc_label_level_inc();
	// This generates a period of about 100us.
	errs	+= aiss8ao8_ao_clk_trig_out	(args->fd, -1, 1, AISS8AO8_AO_CLK_TRIG_OUT_DRIVE, NULL);
	errs	+= aiss8ao8_gen_c_ndiv		(args->fd, -1, 1, ndiv * 4, NULL);
	errs	+= aiss8ao8_data_format		(args->fd, -1, 1, AISS8AO8_DATA_FORMAT_OFF_BIN, NULL);
	buf[0]	= 0x20000;
	sent	= aiss8ao8_write(args->fd, buf, sizeof(buf));
	errs	+= (sent == sizeof(buf)) ? 0 : 1;
	errs	+= aiss8ao8_ao_buf_mode		(args->fd, -1, 1, AISS8AO8_BUF_MODE_CIRC, NULL);
	errs	+= aiss8ao8_gen_c_enable	(args->fd, -1, 1, AISS8AO8_GEN_ENABLE_YES, NULL);
	errs	+= aiss8ao8_ao_enable		(args->fd, -1, 1, AISS8AO8_AIO_ENABLE_YES, NULL);
	errs	+= aiss8ao8_fsamp_ao_report	(args->fd, -1, 1, -1, NULL);

	gsc_label_level_dec();

	// Wait a bit.
	gsc_label("Waiting");
	os_sleep_ms(args->seconds * 1000);
	printf("Done\n");

	return(errs);
}



//*****************************************************************************
static int _input_software_trigger(const args_t* args)
{
	int		errs	= 0;
	time_t	limit;
	time_t	now;
	int		ret;

	gsc_label("Analog Input SW Trigger");
	printf("\n");
	gsc_label_level_inc();
	errs	+= aiss8ao8_ai_clk_trig_out	(args->fd, -1, 1, AISS8AO8_AI_CLK_TRIG_OUT_DRIVE, NULL);
	errs	+= aiss8ao8_gen_b_enable	(args->fd, -1, 1, AISS8AO8_GEN_ENABLE_NO, NULL);
	errs	+= aiss8ao8_ai_burst_enable	(args->fd, -1, 1, AISS8AO8_BURST_ENABLE_YES, NULL);

	// Wait a bit.
	gsc_label("Waiting");
	limit	= time(NULL) + args->seconds;

	for (; errs == 0;)
	{
		ret		= aiss8ao8_ioctl(args->fd, AISS8AO8_IOCTL_AI_SW_TRIGGER, NULL);
		errs	+= ret ? 1 : 0;

		now	= time(NULL);

		if (now >= limit)
			break;

		os_sleep_ms(2);
	}

	if (errs == 0)
		printf("Done\n");

	gsc_label_level_dec();
	return(errs);
}



//*****************************************************************************
static int _output_software_clock(const args_t* args)
{
	int		errs	= 0;
	time_t	limit;
	time_t	now;
	int		ret;

	gsc_label("Analog Output SW Clock");
	printf("\n");
	gsc_label_level_inc();
	errs	+= aiss8ao8_ai_clk_trig_out	(args->fd, -1, 1, AISS8AO8_AI_CLK_TRIG_OUT_DRIVE, NULL);
	errs	+= aiss8ao8_gen_c_enable	(args->fd, -1, 1, AISS8AO8_GEN_ENABLE_NO, NULL);

	// Wait a bit.
	gsc_label("Waiting");
	limit	= time(NULL) + args->seconds;

	for (; errs == 0;)
	{
		ret		= aiss8ao8_ioctl(args->fd, AISS8AO8_IOCTL_AO_SW_CLOCK, NULL);
		errs	+= ret ? 1 : 0;

		now	= time(NULL);

		if (now >= limit)
			break;

		os_sleep_ms(2);
	}

	if (errs == 0)
		printf("Done\n");

	gsc_label_level_dec();
	return(errs);
}



//*****************************************************************************
int perform_tests(const args_t* args)
{
	int	errs	= 0;

	errs	+= aiss8ao8_initialize		(args->fd, -1, 1);
	errs	+= aiss8ao8_ai_clk_trig_out	(args->fd, -1, 1, AISS8AO8_AI_CLK_TRIG_OUT_DRIVE, NULL);

	switch (args->sw_flags)
	{
		default:

			errs	= _continuous_signals(args);
			break;

		case SW_FLAG_INPUT_TRIGGER:

			errs	= _input_software_trigger(args);
			break;

		case SW_FLAG_OUTPUT_CLOCK:

			errs	= _output_software_clock(args);
			break;
	}

	errs	+= aiss8ao8_initialize		(args->fd, -1, 1);
	return(errs);
}



