// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/18AISS8AO8/18AISS8AO8_1.x_GSC_DN/samples/txrate/perform.c $
// $Rev: 52607 $
// $Date: 2023-03-10 14:42:45 -0600 (Fri, 10 Mar 2023) $

// 18AISS8AO8: Sample Application: source file

#include "main.h"



// macros *********************************************************************

#define	_1M	(1024L * 1024L)



// variables ******************************************************************

static u32	_tx[_1M];



//*****************************************************************************
static int _io_mode_select(const args_t* args)
{
	s32	dmdma	= 0;
	int	errs	= 0;

	errs	+= aiss8ao8_query(args->fd, -1, 1, AISS8AO8_QUERY_DMDMA, &dmdma);

	if ((args->io_mode == GSC_IO_MODE_DMDMA) && (dmdma == 0))
	{
		gsc_label_suffix(NULL, NULL);
		printf("Reverting to BMDMA since DMDMA is not supported.\n");
		errs	+= aiss8ao8_tx_io_mode(args->fd, -1, 1, GSC_IO_MODE_BMDMA, NULL);
	}
	else
	{
		errs	+= aiss8ao8_tx_io_mode(args->fd, -1, 1, args->io_mode, NULL);
	}

	return(errs);

}



//*****************************************************************************
static int _rate_test(const args_t* args, long send)
{
	os_time_ns_t	begin;
	os_time_ns_t	end;
	int				errs		= 0;
	long long		data_limit	= args->tx_mb * 1000000;
	long long		data_total	= 0;
	long long		ms;
	long long		ms_limit	= 1000 * args->seconds;
	long long		ms_total	= 0;
	double			rate;
	long			sent;
	long long		total;

	gsc_label("Writing");
	memset(_tx, 0x01, sizeof(_tx));
	os_time_get_ns(&begin);

	for (;;)
	{
		sent	= aiss8ao8_write(args->fd, _tx, send);

		if (sent < 0)
		{
			errs++;
			printf("FAIL <--- (aiss8ao8_write, errno %d)\n", errno);
			break;
		}

		os_time_get_ns(&end);
		data_total	+= sent;
		ms_total	= ((long long) end.tv_sec * 1000 + end.tv_nsec / 1000000)
					- ((long long) begin.tv_sec * 1000 + begin.tv_nsec / 1000000);

		if ((ms_total >= ms_limit) && (data_total >= data_limit))
			break;
	}

	printf(	"%s  (", errs ? "FAIL <---" : "PASS");

	// Bytes
	total	= data_total;
	gsc_label_long_comma(total / 4);
	printf(" Samples, ");

	// Period
	ms	= ms_total;
	gsc_label_float_comma((double) ms / 1000, -1, 3);
	printf(" Seconds, ");

	// Rate
	ms		= (ms <= 0) ? 1 : ms;
	rate	= ((double) total * 1000000 / 4) / ms / 1000;
	gsc_label_float_comma(rate, -1, 0);
	printf(" S/S)\n");

	return(errs);
}



//*****************************************************************************
int perform_tests(const args_t* args)
{
	int		errs	= 0;
	s32		fsamp	= 450000;
	long	send	= sizeof(_tx);

	errs	+= aiss8ao8_query(args->fd, -1, 1, AISS8AO8_QUERY_FSAMP_MAX, &fsamp);
	errs	+= aiss8ao8_config_ao(args->fd, -1, 1, fsamp);
	errs	+= _io_mode_select(args);

	gsc_label ("Write Rate");
	printf("(%d second minimum, %ld MB minimum)\n", (int) args->seconds, (long) args->tx_mb);
	gsc_label_level_inc();

	errs	+= _rate_test(args, send);
	errs	+= aiss8ao8_ao_buf_over_data(args->fd, -1, 1, AISS8AO8_BUF_OVERFLOW_TEST,	NULL);

	gsc_label_level_dec();

	return(errs);
}



