// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/18AISS8AO8/18AISS8AO8_1.x_GSC_DN/utils/ai_buf_underflow.c $
// $Rev: 52611 $
// $Date: 2023-03-10 14:47:31 -0600 (Fri, 10 Mar 2023) $

// 18AISS8AO8: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	aiss8ao8_ai_buf_underflow
*
*	Purpose:
*
*		Provide a visual wrapper for the AISS8AO8_IOCTL_AI_BUF_UNDERFLOW service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int	aiss8ao8_ai_buf_underflow(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr		= buf;
	int			ret;

	if (verbose)
		gsc_label_index("AI Buffer Underflow", index);

	ret		= aiss8ao8_ioctl(fd, AISS8AO8_IOCTL_AI_BUF_UNDERFLOW, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs++;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case -1:

			ptr	= "Feature Not Supported";
			break;

		case AISS8AO8_AI_BUF_UNDERFLOW_NO:

			ptr	= "No";
			break;

		case AISS8AO8_AI_BUF_UNDERFLOW_YES:

			ptr	= "Yes";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


