// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/18AISS8AO8/18AISS8AO8_1.x_GSC_DN/utils/ai_mode.c $
// $Rev: 44668 $
// $Date: 2019-04-04 16:07:30 -0500 (Thu, 04 Apr 2019) $

// 18AISS8AO8: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	aiss8ao8_ai_mode
*
*	Purpose:
*
*		Provide a visual wrapper for the AISS8AO8_IOCTL_AI_MODE service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int	aiss8ao8_ai_mode(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr		= buf;
	int			ret;

	if (verbose)
		gsc_label_index("AI Mode", index);

	ret		= aiss8ao8_ioctl(fd, AISS8AO8_IOCTL_AI_MODE, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs++;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case AISS8AO8_AI_MODE_DIFF:

			ptr	= "Differential";
			break;

		case AISS8AO8_AI_MODE_SINGLE:

			ptr	= "Single Ended";
			break;

		case AISS8AO8_AI_MODE_ZERO:

			ptr	= "Zero Voltage Test";
			break;

		case AISS8AO8_AI_MODE_VREF:

			ptr	= "Vref Test";
			break;

		case AISS8AO8_AI_MODE_AO_0_HI:

			ptr	= "AO Channel 0 High";
			break;

		case AISS8AO8_AI_MODE_AO_1_HI:

			ptr	= "AO Channel 1 High";
			break;

		case AISS8AO8_AI_MODE_AO_2_HI:

			ptr	= "AO Channel 2 High";
			break;

		case AISS8AO8_AI_MODE_AO_3_HI:

			ptr	= "AO Channel 3 High";
			break;

		case AISS8AO8_AI_MODE_AO_4_HI:

			ptr	= "AO Channel 4 High";
			break;

		case AISS8AO8_AI_MODE_AO_5_HI:

			ptr	= "AO Channel 5 High";
			break;

		case AISS8AO8_AI_MODE_AO_6_HI:

			ptr	= "AO Channel 6 High";
			break;

		case AISS8AO8_AI_MODE_AO_7_HI:

			ptr	= "AO Channel 7 High";
			break;

		case AISS8AO8_AI_MODE_AO_0_LO:

			ptr	= "AO Channel 0 Low";
			break;

		case AISS8AO8_AI_MODE_AO_1_LO:

			ptr	= "AO Channel 1 Low";
			break;

		case AISS8AO8_AI_MODE_AO_2_LO:

			ptr	= "AO Channel 2 Low";
			break;

		case AISS8AO8_AI_MODE_AO_3_LO:

			ptr	= "AO Channel 3 Low";
			break;

		case AISS8AO8_AI_MODE_AO_4_LO:

			ptr	= "AO Channel 4 Low";
			break;

		case AISS8AO8_AI_MODE_AO_5_LO:

			ptr	= "AO Channel 5 Low";
			break;

		case AISS8AO8_AI_MODE_AO_6_LO:

			ptr	= "AO Channel 6 Low";
			break;

		case AISS8AO8_AI_MODE_AO_7_LO:

			ptr	= "AO Channel 7 Low";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


