// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/18AISS8AO8/18AISS8AO8_1.x_GSC_DN/utils/ao_range.c $
// $Rev: 44668 $
// $Date: 2019-04-04 16:07:30 -0500 (Thu, 04 Apr 2019) $

// 18AISS8AO8: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	aiss8ao8_ao_range
*
*	Purpose:
*
*		Provide a visual wrapper for the AISS8AO8_IOCTL_AO_RANGE service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int	aiss8ao8_ao_range(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr		= buf;
	int			ret;

	if (verbose)
		gsc_label_index("AO Range", index);

	ret		= aiss8ao8_ioctl(fd, AISS8AO8_IOCTL_AO_RANGE, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs++;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case AISS8AO8_RANGE_2_5V:

			ptr	= "+-2.5 V";
			break;

		case AISS8AO8_RANGE_5V:

			ptr	= "+-5 V";
			break;

		case AISS8AO8_RANGE_10V:

			ptr	= "+-10 V";
			break;

		case AISS8AO8_RANGE_0_5V:

			ptr	= "0 to +5 V";
			break;

		case AISS8AO8_RANGE_0_10V:

			ptr	= "0 to +10 V";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


