// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/18AISS8AO8/18AISS8AO8_1.x_GSC_DN/utils/ao_trig_src.c $
// $Rev: 51968 $
// $Date: 2022-10-26 10:38:14 -0500 (Wed, 26 Oct 2022) $

// 18AISS8AO8: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	aiss8ao8_ao_trig_src
*
*	Purpose:
*
*		Provide a visual wrapper for the AISS8AO8_IOCTL_AO_TRIG_SRC service.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the device to access. Ignore if < 0.
*
*		verbose	0: errors only, !0: all output
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int aiss8ao8_ao_trig_src(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr;
	s32			ret;

	if (verbose)
		gsc_label_index("AO Trigger Source", index);

	ret		= aiss8ao8_ioctl(fd, AISS8AO8_IOCTL_AO_TRIG_SRC, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs	= 1;
			ptr		= buf;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case AISS8AO8_AO_TRIG_SRC_RDG:

			ptr	= "Rate-D Generator";
			break;

		case AISS8AO8_AO_TRIG_SRC_RBG:

			ptr	= "Rate-B Generator";
			break;

		case AISS8AO8_AO_TRIG_SRC_EXT:

			ptr	= "External";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


