// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/18AISS8AO8/18AISS8AO8_1.x_GSC_DN/utils/config_ai.c $
// $Rev: 52611 $
// $Date: 2023-03-10 14:47:31 -0600 (Fri, 10 Mar 2023) $

// 18AISS8AO8: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	aiss8ao8_config_ai
*
*	Purpose:
*
*		Configure the given device's analog input options.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the device to access. Ignore if < 0.
*
*		verbose	Work verbosely?
*
*		fsamp	This is the desired Fsamp rate, or -1 to use the default.
*
*		master	The master clock frequency, or -1.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int aiss8ao8_config_ai(int fd, int index, int verbose, s32 fsamp)
{
	s32	chans	= 0;
	int	errs	= 0;
	s32	mask;
	s32	ndiv;
	s32	under	= 0;
	s32	warp;

	if (verbose)
	{
		gsc_label_index("Input Configuration", index);
		gsc_label_level_inc();
		printf("\n");
	}

	errs	+= aiss8ao8_initialize			(fd, index, verbose);

	errs	+= aiss8ao8_query				(fd, index, verbose, AISS8AO8_QUERY_AI_UNDERFLOW, &under);
	errs	+= aiss8ao8_query				(fd, index, verbose, AISS8AO8_QUERY_CHANNEL_AI_QTY, &chans);
	mask	= (chans == 4) ? 0xF : 0xFF;

	errs	+= aiss8ao8_rx_io_mode			(fd, index, verbose, GSC_IO_MODE_PIO,					NULL);
	errs	+= aiss8ao8_rx_io_overflow		(fd, index, verbose, AISS8AO8_IO_OVERFLOW_CHECK,		NULL);
	errs	+= aiss8ao8_rx_io_timeout		(fd, index, verbose, 30,								NULL);

	if (under)
		errs	+= aiss8ao8_rx_io_underflow	(fd, index, verbose, AISS8AO8_RX_IO_UNDERFLOW_CHECK,	NULL);

	errs	+= aiss8ao8_data_format			(fd, index, verbose, AISS8AO8_DATA_FORMAT_OFF_BIN,		NULL);
	errs	+= aiss8ao8_ai_mode				(fd, index, verbose, AISS8AO8_AI_MODE_DIFF,				NULL);
	errs	+= aiss8ao8_ai_buf_thr_lvl		(fd, index, verbose, 200L * 1024,						NULL);
	errs	+= aiss8ao8_ai_burst_enable		(fd, index, verbose, AISS8AO8_BURST_ENABLE_NO,			NULL);
	errs	+= aiss8ao8_ai_chan_sel			(fd, index, verbose, mask,								NULL);
	errs	+= aiss8ao8_ai_range			(fd, index, verbose, AISS8AO8_RANGE_10V,				NULL);
	errs	+= aiss8ao8_ai_grp_a_filter		(fd, index, verbose, AISS8AO8_AI_GRP_FILTER_NONE,		NULL);
	errs	+= aiss8ao8_ai_grp_b_filter		(fd, index, verbose, AISS8AO8_AI_GRP_FILTER_NONE,		NULL);
	errs	+= aiss8ao8_ai_i_loop_term		(fd, index, verbose, 0,									NULL);
	errs	+= aiss8ao8_ai_clock_src		(fd, index, verbose, AISS8AO8_AI_CLOCK_SRC_RAG,			NULL);

	errs	+= aiss8ao8_ai_clk_trig_out		(fd, index, verbose, AISS8AO8_AI_CLK_TRIG_OUT_OFF,		NULL);
	errs	+= aiss8ao8_fsamp_ai_compute	(fd, index, verbose, fsamp,  &ndiv, &warp,				NULL);
	errs	+= aiss8ao8_gen_a_ndiv			(fd, index, verbose, ndiv,								NULL);
	errs	+= aiss8ao8_warp_mode			(fd, index, verbose, warp,								NULL);
	errs	+= aiss8ao8_gen_a_enable		(fd, index, verbose, AISS8AO8_GEN_ENABLE_YES,			NULL);
	errs	+= aiss8ao8_gen_b_enable		(fd, index, verbose, AISS8AO8_GEN_ENABLE_NO,			NULL);
	errs	+= aiss8ao8_fsamp_ai_report_all	(fd, index, verbose,									NULL);

	errs	+= aiss8ao8_auto_calibrate		(fd, index, verbose);	// Cable may need disconnecting for this.
	errs	+= aiss8ao8_ai_enable			(fd, index, verbose, AISS8AO8_AIO_ENABLE_YES,			NULL);
	errs	+= aiss8ao8_ai_buf_clear		(fd, index, verbose);
	errs	+= aiss8ao8_ai_buf_overflow		(fd, index, verbose, AISS8AO8_BUF_OVERFLOW_CLEAR,		NULL);

	if (verbose)
		gsc_label_level_dec();

	return(errs);
}



