// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/18AISS8AO8/18AISS8AO8_1.x_GSC_DN/utils/config_ao.c $
// $Rev: 51968 $
// $Date: 2022-10-26 10:38:14 -0500 (Wed, 26 Oct 2022) $

// 18AISS8AO8: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	aiss8ao8_config_ao
*
*	Purpose:
*
*		Configure the given device's analog output options.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the device to access. Ignore if < 0.
*
*		verbose	Work verbosely?
*
*		fsamp	This is the desired Fsamp rate, of -1 to use the default.
*
*		master	The master clock frequency, or -1.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int aiss8ao8_config_ao(int fd, int index, int verbose, s32 fsamp)
{
	s32	chans	= 0;
	int	errs	= 0;
	s32	mask;
	s32	ndiv;

	if (verbose)
	{
		gsc_label_index("Output Configuration", index);
		printf("\n");
		gsc_label_level_inc();
	}

	errs	+= aiss8ao8_initialize			(fd, index, verbose);

	if (verbose)
		gsc_label("AO Channel Count");

	errs	+= aiss8ao8_query(fd, -1, 0, AISS8AO8_QUERY_CHANNEL_AO_QTY, &chans);
	mask	= (chans == 4) ? 0xF : 0xFF;

	if (verbose)
		printf("%ld channel%s\n", (long) chans, (chans == 1) ? "" : "s");

	errs	+= aiss8ao8_tx_io_mode			(fd, index, verbose, GSC_IO_MODE_PIO,				NULL);
	errs	+= aiss8ao8_tx_io_overflow		(fd, index, verbose, AISS8AO8_IO_OVERFLOW_CHECK,	NULL);
	errs	+= aiss8ao8_tx_io_timeout		(fd, index, verbose, 30,							NULL);

	errs	+= aiss8ao8_data_format			(fd, index, verbose, AISS8AO8_DATA_FORMAT_OFF_BIN,	NULL);

	errs	+= aiss8ao8_ao_clock_src		(fd, index, verbose, AISS8AO8_AO_CLOCK_SRC_RCG,		NULL);
	errs	+= aiss8ao8_ao_mode				(fd, index, verbose, AISS8AO8_AO_MODE_DIFF,			NULL);
	errs	+= aiss8ao8_ao_buf_mode			(fd, index, verbose, AISS8AO8_BUF_MODE_OPEN,		NULL);
	errs	+= aiss8ao8_ao_output_mode		(fd, index, verbose, AISS8AO8_AO_OUTPUT_MODE_SIMUL,	NULL);
	errs	+= aiss8ao8_ao_buf_thr_lvl		(fd, index, verbose, 200L * 1024,					NULL);
	errs	+= aiss8ao8_ao_burst_enable		(fd, index, verbose, AISS8AO8_BURST_ENABLE_NO,		NULL);
	errs	+= aiss8ao8_ao_chan_sel			(fd, index, verbose, mask,							NULL);
	errs	+= aiss8ao8_ao_range			(fd, index, verbose, AISS8AO8_RANGE_10V,			NULL);
	errs	+= aiss8ao8_ao_clk_trig_out		(fd, index, verbose, AISS8AO8_AO_CLK_TRIG_OUT_OFF,	NULL);

	errs	+= aiss8ao8_fsamp_ao_compute	(fd, index, verbose, fsamp, &ndiv,					NULL);
	errs	+= aiss8ao8_gen_c_ndiv			(fd, index, verbose, ndiv,							NULL);
	errs	+= aiss8ao8_gen_c_enable		(fd, index, verbose, AISS8AO8_GEN_ENABLE_YES,		NULL);
	errs	+= aiss8ao8_gen_d_enable		(fd, index, verbose, AISS8AO8_GEN_ENABLE_NO,		NULL);
	errs	+= aiss8ao8_ao_enable			(fd, index, verbose, AISS8AO8_AIO_ENABLE_YES,		NULL);
	errs	+= aiss8ao8_fsamp_ao_report_all	(fd, index, verbose,								NULL);

	errs	+= aiss8ao8_auto_calibrate		(fd, index, verbose);
	errs	+= aiss8ao8_ao_buf_clear		(fd, index, verbose);
	errs	+= aiss8ao8_ao_buf_over_data	(fd, index, verbose, AISS8AO8_BUF_OVERFLOW_CLEAR,	NULL);
	errs	+= aiss8ao8_ao_buf_over_fram	(fd, index, verbose, AISS8AO8_BUF_OVERFLOW_CLEAR,	NULL);

	if (verbose)
		gsc_label_level_dec();

	return(errs);
}


