// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/18AISS8AO8/18AISS8AO8_1.x_GSC_DN/utils/dio_dir_out.c $
// $Rev: 44668 $
// $Date: 2019-04-04 16:07:30 -0500 (Thu, 04 Apr 2019) $

// 18AISS8AO8: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	aiss8ao8_dio_dir_out
*
*	Purpose:
*
*		Provide a visual wrapper for the AISS8AO8_IOCTL_DIO_DIR_OUT service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int	aiss8ao8_dio_dir_out(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr		= buf;
	int			ret;

	if (verbose)
		gsc_label_index("DIO Outputs", index);

	ret		= aiss8ao8_ioctl(fd, AISS8AO8_IOCTL_DIO_DIR_OUT, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs++;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case AISS8AO8_DIO_DIR_OUT_HI_IN_LO_IN:

			ptr	= "Both Nibbles In";
			break;

		case AISS8AO8_DIO_DIR_OUT_HI_IN_LO_OT:

			ptr	= "Low Nibble Out, High Nibble In";
			break;

		case AISS8AO8_DIO_DIR_OUT_HI_OT_LO_IN:

			ptr	= "Low Nibble In, High Nibble Out";
			break;

		case AISS8AO8_DIO_DIR_OUT_HI_OT_LO_OT:

			ptr	= "Both Nibbles Out";
			break;
	}

	if (get)
		get[0]	= set;

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	return(errs);
}


