// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/18AISS8AO8/18AISS8AO8_1.x_GSC_DN/utils/query.c $
// $Rev: 46488 $
// $Date: 2019-09-12 16:20:05 -0500 (Thu, 12 Sep 2019) $

// 18AISS8AO8: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	aiss8ao8_query
*
*	Purpose:
*
*		Provide a visual wrapper for the AISS8AO8_IOCTL_QUERY service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the query option to access.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int	aiss8ao8_query(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs	= 0;
	const char*	ptr;
	s32			query	= set;
	int			ret;
	s32			tmp;

	switch (query)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Query Error (#%ld)", (long) query);
			break;

		case AISS8AO8_QUERY_AI_UNDERFLOW:	ptr	= "AI Underflow";			break;
		case AISS8AO8_QUERY_AUTO_CAL_MS:	ptr	= "Auto-Cal Duration";		break;
		case AISS8AO8_QUERY_CHANNEL_AI_MAX:	ptr	= "Max AI Channels";		break;
		case AISS8AO8_QUERY_CHANNEL_AI_QTY:	ptr	= "Installed AI Channels";	break;
		case AISS8AO8_QUERY_CHANNEL_AO_MAX:	ptr	= "Max AO Channels";		break;
		case AISS8AO8_QUERY_CHANNEL_AO_QTY:	ptr	= "Installed AO Channels";	break;
		case AISS8AO8_QUERY_COUNT:			ptr	= "Query Options";			break;
		case AISS8AO8_QUERY_DEVICE_TYPE:	ptr	= "Device Type";			break;
		case AISS8AO8_QUERY_DMDMA:			ptr	= "Demand Mode DMA";		break;
		case AISS8AO8_QUERY_FIFO_SIZE_RX:	ptr	= "Rx FIFO Size";			break;
		case AISS8AO8_QUERY_FIFO_SIZE_TX:	ptr	= "Tx FIFO Size";			break;
		case AISS8AO8_QUERY_FILTER_A:		ptr	= "Filter A";				break;
		case AISS8AO8_QUERY_FILTER_B:		ptr	= "Filter B";				break;
		case AISS8AO8_QUERY_FSAMP_MAX:		ptr	= "Fsamp Max";				break;
		case AISS8AO8_QUERY_FSAMP_MIN:		ptr	= "Fsamp Min";				break;
		case AISS8AO8_QUERY_INIT_MS:		ptr	= "Init. Duration";			break;
		case AISS8AO8_QUERY_MASTER_CLOCK:	ptr	= "Master Clock";			break;
		case AISS8AO8_QUERY_NDIV_MAX:		ptr	= "Ndiv Maximum";			break;
		case AISS8AO8_QUERY_NDIV_MIN:		ptr	= "Ndiv Minimum";			break;
	}

	if (verbose)
		gsc_label_index(ptr, index);

	ret		= aiss8ao8_ioctl(fd, AISS8AO8_IOCTL_QUERY, &set);
	errs	+= ret ? 1 : 0;

	switch (query)
	{
		default:

			errs++;
			sprintf(buf, "%ld", (long) set);
			break;

		case AISS8AO8_QUERY_AI_UNDERFLOW:
		case AISS8AO8_QUERY_DMDMA:

			strcpy(buf, set ? "Supported" : "Not Supported");
			break;

		case AISS8AO8_QUERY_CHANNEL_AI_MAX:
		case AISS8AO8_QUERY_CHANNEL_AI_QTY:
		case AISS8AO8_QUERY_CHANNEL_AO_MAX:
		case AISS8AO8_QUERY_CHANNEL_AO_QTY:
		case AISS8AO8_QUERY_COUNT:
		case AISS8AO8_QUERY_NDIV_MAX:
		case AISS8AO8_QUERY_NDIV_MIN:

			gsc_label_long_comma_buf(set, buf);
			break;

		case AISS8AO8_QUERY_MASTER_CLOCK:
		case AISS8AO8_QUERY_FILTER_A:
		case AISS8AO8_QUERY_FILTER_B:

			if (set == -1)
			{
				strcpy(buf, "Not Known");
			}
			else
			{
				gsc_label_long_comma_buf(set, buf);
				strcat(buf, " Hz");
			}

			break;

		case AISS8AO8_QUERY_AUTO_CAL_MS:
		case AISS8AO8_QUERY_INIT_MS:

			gsc_label_long_comma_buf(set, buf);
			strcat(buf, " ms");
			break;

		case AISS8AO8_QUERY_FSAMP_MAX:
		case AISS8AO8_QUERY_FSAMP_MIN:

			gsc_label_long_comma_buf(set, buf);
			strcat(buf, " S/S");
			break;

		case AISS8AO8_QUERY_FIFO_SIZE_RX:
		case AISS8AO8_QUERY_FIFO_SIZE_TX:

			ptr	= "";
			tmp	= set;

			if ((tmp) && ((tmp % 1024) == 0))
			{
				ptr	= "K";
				tmp	/= 1024;
			}

			if ((tmp) && ((tmp % 1024) == 0))
			{
				ptr	= "M";
				tmp	/= 1024;
			}

			sprintf(buf, "%ld%s Words", (long) tmp, ptr);
			break;

		case AISS8AO8_QUERY_DEVICE_TYPE:

			if (set == GSC_DEV_TYPE_18AISS8AO8)
			{
				strcpy(buf, "18AISS8AO8");
			}
			else
			{
				errs++;
				sprintf(buf, "INVALID: %ld", (long) set);
			}

			break;
	}

	if (verbose)
	{
		if (errs)
			printf("FAIL <---  (%s)\n", buf);
		else
			printf("%s\n", buf);
	}

	if (get)
		get[0]	= set;

	return(errs);
}


