// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/18AISS8AO8/18AISS8AO8_1.x_GSC_DN/utils/reg.c $
// $Rev: 51968 $
// $Date: 2022-10-26 10:38:14 -0500 (Wed, 26 Oct 2022) $

// 18AISS8AO8: Utilities: source file

#include "main.h"



// macros *********************************************************************

#define	_GSC_REG(a)							"GSC " #a, AISS8AO8_GSC_##a, 0, 0
#define	WIDTH								22

#define	SHOW_FIELD(b,e,eof,list,desc)		gsc_reg_field_show(width + 10, WIDTH, value,(b),(e),(eof),(list),(desc))



//*****************************************************************************
static int _bcr_decode(int fd, int supported, u32 value, int width)
{
	static const char*	clear[]		= { "Clear",			"Set"					};
	static const char*	disabled[]	= { "Disabled",			"Enabled"				};
	static const char*	idle[]		= { "Idle",				"Active"				};
	static const char*	immed[]		= { "No (immediate)",	"Yes (simultaneous)"	};
	static const char*	no[]		= { "No",				"Yes"					};
	static const char*	twos[]		= { "Twos Compliment",	"Offset Binary"			};

	SHOW_FIELD( 7,  0, 1, NULL,		"Reserved"				);
	SHOW_FIELD( 8,  8, 1, idle,		"Input SW Clock"		);
	SHOW_FIELD( 9,  9, 1, disabled,	"Enable Input Burst"	);
	SHOW_FIELD(10, 10, 1, idle,		"Input Burst Busy"		);
	SHOW_FIELD(11, 11, 1, idle,		"Input SW Trigger"		);
	SHOW_FIELD(12, 12, 1, disabled,	"Enable Input Buffer"	);
	SHOW_FIELD(13, 13, 1, idle,		"Clear Input Buffer"	);
	SHOW_FIELD(14, 14, 1, clear,	"AI Threshold Flag"		);
	SHOW_FIELD(15, 15, 1, no,		"Input Overflow"		);
	SHOW_FIELD(17, 16, 1, NULL,		"Reserved"				);
	SHOW_FIELD(18, 18, 1, immed,	"Simul. Outputs"		);
	SHOW_FIELD(19, 19, 1, NULL,		"Reserved"				);
	SHOW_FIELD(20, 20, 1, idle,		"Output SW Clock"		);
	SHOW_FIELD(21, 21, 1, disabled,	"Rate-C Generator"		);
	SHOW_FIELD(22, 22, 1, disabled,	"Rate-D Generator"		);
	SHOW_FIELD(23, 23, 1, idle,		"Output SW Trigger"		);
	SHOW_FIELD(24, 24, 1, NULL,		"Reserved"				);
	SHOW_FIELD(25, 25, 1, twos,		"Data Format"			);
	SHOW_FIELD(26, 26, 1, disabled,	"Rate-A Generator"		);
	SHOW_FIELD(27, 27, 1, disabled,	"Rate-B Generator"		);
	SHOW_FIELD(28, 28, 1, idle,		"Auto-Cal"				);
	SHOW_FIELD(29, 29, 1, no,		"Auto-Cal Pass"			);
	SHOW_FIELD(30, 30, 1, disabled,	"AI Warp Mode"			);
	SHOW_FIELD(31, 31, 1, idle,		"Initialize"			);
	return(0);
}



//*****************************************************************************
static int _diopr_decode(int fd, int supported, u32 value, int width)
{
	static const char*	input[]		= { "Input",	"Output"	};

	SHOW_FIELD( 3,  0, 1, NULL,		"Low Nibble Data"		);
	SHOW_FIELD( 6,  4, 1, NULL,		"Reserved"				);
	SHOW_FIELD( 7,  7, 1, input,	"Low Nibble Direction"	);
	SHOW_FIELD(11,  8, 1, NULL,		"High Nibble Data"		);
	SHOW_FIELD(14, 12, 1, NULL,		"Reserved"				);
	SHOW_FIELD(15, 15, 1, input,	"High Nibble Direction"	);
	SHOW_FIELD(31, 16, 1, NULL,		"Reserved"				);
	return(0);
}



//*****************************************************************************
static int _clsr_decode(int fd, int supported, u32 value, int width)
{
	static const char*	disable[]	= { "Disabled",	"Enabled"	};

	SHOW_FIELD( 0,  0, 1, disable,	"Terminator 0"	);
	SHOW_FIELD( 1,  1, 1, disable,	"Terminator 1"	);
	SHOW_FIELD( 2,  2, 1, disable,	"Terminator 2"	);
	SHOW_FIELD( 3,  3, 1, disable,	"Terminator 3"	);
	SHOW_FIELD( 4,  4, 1, disable,	"Terminator 4"	);
	SHOW_FIELD( 5,  5, 1, disable,	"Terminator 5"	);
	SHOW_FIELD( 6,  6, 1, disable,	"Terminator 6"	);
	SHOW_FIELD( 7,  7, 1, disable,	"Terminator 7"	);
	SHOW_FIELD(31,  8, 1, NULL,		"Reserved"		);
	return(0);
}



//*****************************************************************************
static int _icr_decode(int fd, int supported, u32 value, int width)
{
	static const char*	config[]	=
	{
		"Differential",		// 0
		"Single-Ended",		// 1
		"Zero Selftest",	// 2
		"+VREF Selftest",	// 3
		"RESERVED",			// 4
		"RESERVED",			// 5
		"RESERVED",			// 6
		"RESERVED",			// 7
		"RESERVED",			// 8
		"RESERVED",			// 9
		"RESERVED",			// 10
		"RESERVED",			// 11
		"RESERVED",			// 12
		"RESERVED",			// 13
		"RESERVED",			// 14
		"RESERVED",			// 15
		"Output Channel 0 High",	// 16
		"Output Channel 1 High",	// 17
		"Output Channel 2 High",	// 18
		"Output Channel 3 High",	// 19
		"Output Channel 4 High",	// 20
		"Output Channel 5 High",	// 21
		"Output Channel 6 High",	// 22
		"Output Channel 7 High",	// 23
		"Output Channel 0 Low",		// 24
		"Output Channel 1 Low",		// 25
		"Output Channel 2 Low",		// 26
		"Output Channel 3 Low",		// 27
		"Output Channel 4 Low",		// 28
		"Output Channel 5 Low",		// 29
		"Output Channel 6 Low",		// 30
		"Output Channel 7 Low"		// 31
	};

	static const char*	clk_src[]	=
	{
		"Rate-A Generator",
		"Rate-C Generator",
		"AI External Clock Input",
		"RESERVED"
	};

	static const char*	filter[]	=
	{
		"None",
		"Filter-A",
		"Filter-B",
		"RESERVED"
	};

	static const char*	trg_src[]	=
	{
		"Rate-B Generator",
		"Rate-D Generator",
		"AI External Trigger Input",
		"RESERVED"
	};

	static const char*	range[]	=
	{
		"+-2.5 Volts",
		"+-5.0 Volts",
		"+-10 Volts",
		"0 - +5 Volts",
		"0 - +10 Volts",
		"RESERVED",
		"RESERVED",
		"RESERVED"
	};

	static const char*	disabled[]	= { "Disabled",	"Enabled"	};

	SHOW_FIELD( 0,  0, 1, disabled,	"Input 0"			);
	SHOW_FIELD( 1,  1, 1, disabled,	"Input 1"			);
	SHOW_FIELD( 2,  2, 1, disabled,	"Input 2"			);
	SHOW_FIELD( 3,  3, 1, disabled,	"Input 3"			);
	SHOW_FIELD( 4,  4, 1, disabled,	"Input 4"			);
	SHOW_FIELD( 5,  5, 1, disabled,	"Input 5"			);
	SHOW_FIELD( 6,  6, 1, disabled,	"Input 6"			);
	SHOW_FIELD( 7,  7, 1, disabled,	"Input 7"			);
	SHOW_FIELD(10,  8, 1, range,	"Input Range"		);
	SHOW_FIELD(11, 11, 1, disabled,	"Clk Trig Output"	);
	SHOW_FIELD(13, 12, 1, clk_src,	"Clock Source"		);
	SHOW_FIELD(15, 14, 1, trg_src,	"Trigger Source"	);
	SHOW_FIELD(20, 16, 1, config,	"AI Mode"			);
	SHOW_FIELD(23, 21, 1, NULL,		"Reserved"			);
	SHOW_FIELD(25, 24, 1, filter,	"Group-A Filter"	);
	SHOW_FIELD(27, 26, 1, filter,	"Group-A Filter"	);
	SHOW_FIELD(31, 28, 1, NULL,		"Reserved"			);
	return(0);
}



//*****************************************************************************
static int _ibdr_decode(int fd, int supported, u32 value, int width)
{
	static const char*	no[]	= { "No",	"Yes"	};

	SHOW_FIELD(17,  0, 1, NULL,	"Data"				);
	SHOW_FIELD(18, 18, 1, no,	"End of Burst"		);
	SHOW_FIELD(19, 19, 1, NULL,	"Reserved"			);
	SHOW_FIELD(22, 20, 1, NULL,	"Channel Number"	);
	SHOW_FIELD(31, 23, 1, NULL,	"Reserved"			);
	return(0);
}



//*****************************************************************************
static int _rgxr_decode(int fd, int supported, u32 value, int width)
{
	u32	field;

	SHOW_FIELD(23,  0, 0, NULL,	"Ndiv"		);
	field	= value & 0x00FFFFFF;
	printf("%ld\n", (long) field);

	SHOW_FIELD(31, 24, 1, NULL,	"Reserved"	);
	return(0);
}



//*****************************************************************************
static int _ibbsr_decode(int fd, int supported, u32 value, int width)
{
	u32	field;

	SHOW_FIELD(23,  0, 0, NULL,	"Size"		);
	field	= value & 0x00FFFFFF;
	printf("%ld\n", (long) field);

	SHOW_FIELD(31, 24, 1, NULL,	"Reserved"	);
	return(0);
}



//*****************************************************************************
static int _xbsr_decode(int fd, int supported, u32 value, int width)
{
	u32	field;

	SHOW_FIELD(19,  0, 0, NULL,	"Fill Level"	);
	field	= value & 0xFFFFF;
	printf("%ld Samples\n", (long) field);

	SHOW_FIELD(31, 20, 1, NULL,	"Reserved"		);

	return(0);
}



//*****************************************************************************
static int _xbtr_decode(int fd, int supported, u32 value, int width)
{
	static const char*	clear[]	= { "Clear",	"Set"	};

	u32	field;

	SHOW_FIELD(19,  0, 0, NULL,		"Threshold Level"	);
	field	= value & 0xFFFFF;
	printf("%ld Samples\n", (long) field);

	SHOW_FIELD(20, 20, 1, clear,	"Threshold Flag"	);
	SHOW_FIELD(31, 21, 1, NULL,		"Reserved"			);

	return(0);
}



//*****************************************************************************
static int _psr_decode(int fd, int supported, u32 value, int width)
{
	static const char*	idle[]		= { "Idle",		"Active"	};
	static const char*	ignore[]	= { "Ignore",	"Selected"	};

	SHOW_FIELD( 0,  0, 1, ignore,	"Auto Cal Done"			);
	SHOW_FIELD( 1,  1, 1, ignore,	"AI Thresh. High->Low"	);
	SHOW_FIELD( 2,  2, 1, ignore,	"AI Thresh. Low->High"	);
	SHOW_FIELD( 3,  3, 1, ignore,	"Input Overun"			);
	SHOW_FIELD( 4,  4, 1, ignore,	"AI Burst Initiated"	);
	SHOW_FIELD( 5,  5, 1, ignore,	"AI Burst Completed"	);
	SHOW_FIELD( 6,  6, 1, ignore,	"Analog Input Clock"	);
	SHOW_FIELD( 7,  7, 1, ignore,	"Analog Output Clock"	);
	SHOW_FIELD( 8,  8, 1, ignore,	"DIO 0 Low->High"		);
	SHOW_FIELD( 9,  9, 1, ignore,	"AO Thresh. High->Low"	);
	SHOW_FIELD(10, 10, 1, ignore,	"AO Thresh. Low->High"	);
	SHOW_FIELD(11, 11, 1, ignore,	"Load Ready High->Low"	);
	SHOW_FIELD(12, 12, 1, ignore,	"Load Ready Low->High"	);
	SHOW_FIELD(13, 13, 1, ignore,	"Output Burst Ready"	);
	SHOW_FIELD(14, 14, 1, ignore,	"Output Overflow"		);
	SHOW_FIELD(15, 15, 1, NULL,		"Reserved"				);

	SHOW_FIELD(16, 16, 1, idle,		"Auto Cal Done"			);
	SHOW_FIELD(17, 17, 1, idle,		"AI Thresh. High->Low"	);
	SHOW_FIELD(18, 18, 1, idle,		"AI Thresh. Low->High"	);
	SHOW_FIELD(19, 19, 1, idle,		"Input Overun"			);
	SHOW_FIELD(20, 20, 1, idle,		"AI Burst Initiated"	);
	SHOW_FIELD(21, 21, 1, idle,		"AI Burst Completed"	);
	SHOW_FIELD(22, 22, 1, idle,		"Analog Input Clock"	);
	SHOW_FIELD(23, 23, 1, idle,		"Analog Output Clock"	);
	SHOW_FIELD(24, 24, 1, idle,		"DIO 0 Low->High"		);
	SHOW_FIELD(25, 25, 1, idle,		"AO Thresh. High->Low"	);
	SHOW_FIELD(26, 26, 1, idle,		"AO Thresh. Low->High"	);
	SHOW_FIELD(27, 27, 1, idle,		"Load Ready High->Low"	);
	SHOW_FIELD(28, 28, 1, idle,		"Load Ready Low->High"	);
	SHOW_FIELD(29, 29, 1, idle,		"Output Burst Ready"	);
	SHOW_FIELD(30, 30, 1, idle,		"Output Overflow"		);
	SHOW_FIELD(31, 31, 1, NULL,		"Reserved"				);
	return(0);
}



//*****************************************************************************
static int _acr_decode(int fd, int supported, u32 value, int width)
{
	static const char*	filter[]	=
	{
		"A=80kHz, B=200kHz",
		"RESERVED",
		"RESERVED",
		"RESERVED"
	};

	static const char*	master[]	=
	{
		"40.320 MHz",
		"RESERVED",
		"RESERVED",
		"RESERVED"
	};

	static const char*	chans[]	= { "8 Channels",	"4 Channels"	};
	static const char*	mem[]	= { "4MB",			"2MB"			};

	u32	field;

	SHOW_FIELD(11,  0, 0, NULL,		"Firmware Revision"	);
	field	= value & 0xFFF;
	printf("%03lX\n", (long) field);

	SHOW_FIELD(15, 12, 1, NULL,		"Reserved"			);
	SHOW_FIELD(16, 16, 1, chans,	"AI & AO Channels"	);
	SHOW_FIELD(17, 17, 1, mem,		"Total FIFO Size"	);
	SHOW_FIELD(19, 18, 1, filter,	"AI Filters"		);
	SHOW_FIELD(21, 20, 1, master,	"Master Clock"		);
	SHOW_FIELD(31, 22, 1, NULL,		"Reserved"			);
	return(0);
}



//*****************************************************************************
static int _boor_decode(int fd, int supported, u32 value, int width)
{
	static const char*	clear[]		= { "Clear",	"Set"				};
	static const char*	disable[]	= { "Disabled",	"Enabled"			};
	static const char*	idle[]		= { "Idle",		"Active"			};
	static const char*	no[]		= { "No",		"Yes"				};

	SHOW_FIELD( 4,  0, 1, NULL,		"Reserved"				);
	SHOW_FIELD( 5,  5, 1, disable,	"Output Clocking"		);
	SHOW_FIELD( 6,  6, 1, no,		"Output Clock Ready"	);
	SHOW_FIELD( 7,  7, 1, idle,		"Output SW Clock"		);
	SHOW_FIELD( 8,  8, 1, no,		"Circular Buffer"		);
	SHOW_FIELD( 9,  9, 1, idle,		"Load Request"			);
	SHOW_FIELD(10, 10, 1, no,		"Load Ready"			);
	SHOW_FIELD(11, 11, 1, idle,		"AO Buffer Clear"		);
	SHOW_FIELD(12, 12, 1, no,		"AO Buffer Empty"		);
	SHOW_FIELD(13, 13, 1, clear,	"Buffer Thresh. Flag"	);
	SHOW_FIELD(14, 14, 1, NULL,		"Reserved"				);
	SHOW_FIELD(15, 15, 1, no,		"AO Buffer Full"		);
	SHOW_FIELD(16, 16, 1, no,		"AO Buffer Overflow"	);
	SHOW_FIELD(17, 17, 1, no,		"AO Frame Overflow"		);
	SHOW_FIELD(18, 18, 1, no,		"Burst Ready"			);
	SHOW_FIELD(19, 19, 1, no,		"Output Burst Enable"	);
	SHOW_FIELD(20, 20, 1, idle,		"Output SW Trigger"		);
	SHOW_FIELD(31, 21, 1, NULL,		"Reserved"				);
	return(0);
}



//*****************************************************************************
static int _obdr_decode(int fd, int supported, u32 value, int width)
{
	static const char*	no[]	= { "No",	"Yes"	};

	SHOW_FIELD(17,  0, 1, NULL,	"Data"		);
	SHOW_FIELD(18, 18, 1, no,	"EOF Flag"	);
	SHOW_FIELD(31, 19, 1, NULL,	"Reserved"	);
	return(0);
}



//*****************************************************************************
static int _ocr_decode(int fd, int supported, u32 value, int width)
{
	static const char*	clk_src[]	=
	{
		"Rate-C Generator",
		"Rate-A Generator",
		"AI External Clock Input",
		"RESERVED"
	};

	static const char*	range[]	=
	{
		"+-2.5 Volts",
		"+-5.0 Volts",
		"+-10 Volts",
		"0 - +5 Volts",
		"0 - +10 Volts",
		"RESERVED",
		"RESERVED",
		"RESERVED"
	};

	static const char*	trg_src[]	=
	{
		"Rate-D Generator",
		"Rate-B Generator",
		"AI External Trigger Input",
		"RESERVED"
	};

	static const char*	config[]	= { "Single-Ended",	"Differential"	};
	static const char*	disabled[]	= { "Disabled",		"Enabled"		};

	SHOW_FIELD( 0,  0, 1, disabled,	"Output 0"			);
	SHOW_FIELD( 1,  1, 1, disabled,	"Output 1"			);
	SHOW_FIELD( 2,  2, 1, disabled,	"Output 2"			);
	SHOW_FIELD( 3,  3, 1, disabled,	"Output 3"			);
	SHOW_FIELD( 4,  4, 1, disabled,	"Output 4"			);
	SHOW_FIELD( 5,  5, 1, disabled,	"Output 5"			);
	SHOW_FIELD( 6,  6, 1, disabled,	"Output 6"			);
	SHOW_FIELD( 7,  7, 1, disabled,	"Output 7"			);
	SHOW_FIELD(10,  8, 1, range,	"Output Range"		);
	SHOW_FIELD(11, 11, 1, disabled,	"Clk Trig Output"	);
	SHOW_FIELD(13, 12, 1, clk_src,	"Clock Source"		);
	SHOW_FIELD(15, 14, 1, trg_src,	"Trigger Source"	);
	SHOW_FIELD(16, 16, 1, config,	"AO Mode"			);
	SHOW_FIELD(31, 17, 1, NULL,		"Reserved"			);
	return(0);
}



// variables ******************************************************************

static gsc_reg_def_t	_gsc[]	=
{
	{ _GSC_REG(BCR),		0,	_bcr_decode,	"Board Control Register"				},
	{ _GSC_REG(DIOPR),		0,	_diopr_decode,	"Digital I/O Port Register"				},
	{ _GSC_REG(CLSR),		0,	_clsr_decode,	"Current Loop Selection Register"		},
	{ _GSC_REG(ICR),		0,	_icr_decode,	"Input Configuration Register"			},
	{ _GSC_REG(IBDR),		0,	_ibdr_decode,	"Input Buffer Data Register"			},
	{ _GSC_REG(RAGR),		0,	_rgxr_decode,	"Rate-A Generator Register"				},
	{ _GSC_REG(RBGR),		0,	_rgxr_decode,	"Rate-B Generator Register"				},
	{ _GSC_REG(IBBSR),		0,	_ibbsr_decode,	"Input Burst Block Size Register"		},
	{ _GSC_REG(IBSR),		0,	_xbsr_decode,	"Input Buffer Size Register"			},
	{ _GSC_REG(IBTR),		0,	_xbtr_decode,	"Input Buffer Threshold Register"		},
	{ _GSC_REG(PSR),		0,	_psr_decode,	"Primary Status Register"				},
	{ _GSC_REG(ACR),		0,	_acr_decode,	"Assembly Configuration Register"		},
	{ _GSC_REG(AVR),		0,	NULL,			"Autocal Values Register"				},
	{ _GSC_REG(BOOR),		0,	_boor_decode,	"Buffered Output Operations Register"	},
	{ _GSC_REG(OBTR),		0,	_xbtr_decode,	"Output Buffer Threshold Register"		},
	{ _GSC_REG(OBSR),		0,	_xbsr_decode,	"Output Buffer Size Register"			},
	{ _GSC_REG(OBDR),		0,	_obdr_decode,	"Output Buffer Data Register"			},
	{ _GSC_REG(RCGR),		0,	_rgxr_decode,	"Rate-C Generator Register"				},
	{ _GSC_REG(RDGR),		0,	_rgxr_decode,	"Rate-D Generator Register"				},
	{ _GSC_REG(OCR),		0,	_ocr_decode,	"Output Configuration Register"			},

	{ NULL, 0, 0, 0, 0,		NULL	}
};



//*****************************************************************************
static const gsc_reg_def_t* _find_reg(u32 reg, const gsc_reg_def_t* list)
{
	const gsc_reg_def_t*	def	= NULL;
	int						i;

	for (i = 0; list[i].name; i++)
	{
		if (reg == list[i].reg)
		{
			def	= &list[i];
			break;
		}
	}

	return(def);
}



/******************************************************************************
*
*	Function:	aiss8ao8_reg_get_def_id
*
*	Purpose:
*
*		Retrieve the register definition structure given the register id.
*
*	Arguments:
*
*		reg		The id of the register to access.
*
*	Returned:
*
*		NULL	The register id wasn't found.
*		else	A pointer to the register definition.
*
******************************************************************************/

const gsc_reg_def_t* aiss8ao8_reg_get_def_id(u32 reg)
{
	const gsc_reg_def_t*	def;

	def	= _find_reg(reg, _gsc);
	return(def);
}



/******************************************************************************
*
*	Function:	aiss8ao8_reg_get_def_index
*
*	Purpose:
*
*		Retrieve the register definition structure based on an index.
*
*	Arguments:
*
*		index	The index of the register to access.
*
*	Returned:
*
*		NULL	The index doesn't correspond to a known register.
*		else	A pointer to the register definition.
*
******************************************************************************/

const gsc_reg_def_t* aiss8ao8_reg_get_def_index(int index)
{
	const gsc_reg_def_t*	def;

	if (index < 0)
		def	= NULL;
	else if (index >= (SIZEOF_ARRAY(_gsc) - 1))
		def	= NULL;
	else
		def	= &_gsc[index];

	return(def);
}



/******************************************************************************
*
*	Function:	aiss8ao8_reg_get_desc
*
*	Purpose:
*
*		Retrieve the description of the specified register.
*
*	Arguments:
*
*		reg		The register whose description is desired.
*
*	Returned:
*
*		!NULL	The register's name.
*
******************************************************************************/

const char* aiss8ao8_reg_get_desc(u32 reg)
{
	const gsc_reg_def_t*	def;
	const char*				desc;

	def	= _find_reg(reg, _gsc);

	if (def)
		desc	= def->desc;
	else
		desc	= "UNKNOWN";

	return(desc);
}



/******************************************************************************
*
*	Function:	aiss8ao8_reg_get_name
*
*	Purpose:
*
*		Retrieve the name of the specified register.
*
*	Arguments:
*
*		reg		The register whose name is desired.
*
*	Returned:
*
*		!NULL	The register's name.
*
******************************************************************************/

const char* aiss8ao8_reg_get_name(u32 reg)
{
	const gsc_reg_def_t*	def;
	const char*				name;

	def	= _find_reg(reg, _gsc);

	if (def)
		name	= def->name;
	else
		name	= "UNKNOWN";

	return(name);
}



/******************************************************************************
*
*	Function:	aiss8ao8_reg_list
*
*	Purpose:
*
*		List the GSC registers and their values.
*
*	Arguments:
*
*		fd		The handle to access the device.
*
*		detail	List the register details?
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int aiss8ao8_reg_list(int fd, int detail)
{
	int	errs;

	errs	= gsc_reg_list(fd, _gsc, detail, aiss8ao8_reg_read);
	return(errs);
}


